import re
import os

import constants as c
import ui as ui
import utils as utils


class Scan(object):
    """
    Loop over each role on the roles path and report back stats on them.
    """
    def __init__(self, args, options, rebuild=False, dump=False):
        if len(args) == 0:
            self.roles_path = os.path.join(os.getcwd(), c.ROLES_PATH)
        else:
            self.roles_path = args[0]

        self.options = options
        self.rebuild = rebuild
        self.dump = dump

        repo_prefix = ""
        if rebuild:
            repo_prefix = options.repo_prefix

        self.roles = utils.roles_dict(self.roles_path, repo_prefix)

        if self.options.limit:
            self.limit_roles()

        utils.exit_if_no_roles(len(self.roles.keys()), self.roles_path)

        self.regex_facts = re.compile(r"set_fact:\w+")
        self.paths = {}
        self.all_files = []
        self.yaml_files = []
        self.defaults = ""
        self.facts = []
        self.dependencies = []
        self.verify = False

        # only load and validate the templates when rebuilding files
        if self.rebuild:
            self.meta_template = utils.template(options.template_meta)
            self.readme_template = utils.template(options.template_readme)
            self.verify = True
            self.meta_dict = None

        self.report = {
            "totals": {
                "roles": 0,
                "dependencies": 0,
                "defaults": 0,
                "facts": 0,
                "files": 0,
                "lines": 0,
            },
            "state": {
                "ok_meta": 0,
                "ok_readme": 0,
                "skipped_meta": 0,
                "skipped_readme": 0,
                "changed_meta": 0,
                "changed_readme": 0,
                "failed_meta": 0,
                "failed_readme": 0
            },
            "roles": {},
            "stats": {
                "longest_role_name_length": len(max(self.roles, key=len))
            }
        }

        self.scan_roles()
        if self.dump:
            self.dump_roles()

    def limit_roles(self):
        """
        Limit the roles being scanned.
        """
        new_roles = {}
        roles = self.options.limit.split(",")

        for key, value in self.roles.iteritems():
            for role in roles:
                role = role.strip()
                if key == role:
                    new_roles[key] = value

        self.roles = new_roles

    def scan_roles(self):
        """
        Iterate over each role and report its stats.
        """
        for key, value in sorted(self.roles.iteritems()):
            self.paths["role"] = os.path.join(self.roles_path, key)
            self.paths["meta"] = os.path.join(self.paths["role"],
                                              "meta", "main.yml")
            self.paths["readme"] = os.path.join(self.paths["role"],
                                                "README.md")
            self.paths["defaults"] = os.path.join(self.paths["role"],
                                                  "defaults", "main.yml")

            self.report["roles"][key] = self.report_role(key)

            # we are writing a meta and readme file which means the
            # the state of the role needs to be updated before it gets
            # output by the ui
            if self.verify:
                if self.valid_meta(key) and self.rebuild:
                    self.set_meta_template_vars(value)
                    self.write_meta(key)
                    self.set_readme_template_vars(key, value)
                    self.write_readme(key)

            if not self.options.quiet:
                ui.role(key,
                        self.report["roles"][key],
                        self.report["stats"]["longest_role_name_length"])

        self.tally_role_columns()

        if not self.options.quiet:
            ui.totals(self.report["totals"],
                      len(self.report["roles"].keys()),
                      self.report["stats"]["longest_role_name_length"])

            if self.verify:
                print "\n"
                ui.state_totals(self.report["state"], "meta",
                                self.report["stats"]
                                ["longest_role_name_length"])
                ui.state_totals(self.report["state"], "readme",
                                self.report["stats"]
                                ["longest_role_name_length"])

    def dump_roles(self):
        """
        Write the contents of the report to json.
        """
        out_file = self.options.out_file

        if not out_file.endswith(".json"):
            out_file += ".json"

        # remove unnecessary data and convert string dumps to yaml
        del self.report["state"]
        for role in self.report["roles"]:
            del self.report["roles"][role]["state"]
            if not self.options.with_readme:
                del self.report["roles"][role]["readme"]

            defaults = self.report["roles"][role]["defaults"]
            self.report["roles"][role]["defaults"] = utils.yaml_load("",
                                                                     defaults)
            meta = self.report["roles"][role]["meta"]
            self.report["roles"][role]["meta"] = utils.yaml_load("", meta)

        report_as_json_string = utils.dict_to_json(self.report)
        utils.string_to_file(out_file, report_as_json_string)

        if not self.options.quiet:
            ui.ok("", c.MESSAGES["dump_success"], out_file)

    def report_role(self, role):
        """
        Return the fields gathered.
        """
        self.yaml_files = []

        fields = {
            "state": "skipped",
            "total_files": self.gather_files(),
            "total_lines": self.gather_lines(),
            "total_facts": self.gather_facts(),
            "total_defaults": self.gather_defaults(),
            "facts": self.facts,
            "defaults": self.defaults,
            "meta": self.gather_meta(),
            "readme": self.gather_readme(),
            "dependencies": self.dependencies,
            "total_dependencies": len(self.dependencies)
        }

        return fields

    def gather_meta(self):
        """
        Return the meta file.
        """
        if not os.path.exists(self.paths["meta"]):
            return ""

        meta = utils.file_to_string(self.paths["meta"])

        meta_dict = utils.yaml_load(self.paths["meta"])

        if meta_dict and "dependencies" in meta_dict:
            self.dependencies = meta_dict["dependencies"]
        else:
            self.dependencies = []

        return meta

    def gather_readme(self):
        """
        Return the readme file.
        """
        if not os.path.exists(self.paths["readme"]):
            return ""

        return utils.file_to_string(self.paths["readme"])

    def gather_defaults(self):
        """
        Return the number of default variables.
        """
        total_defaults = 0
        defaults_lines = []

        if not os.path.exists(self.paths["defaults"]):
            return 0

        file = open(self.paths["defaults"], "r")
        for line in file:
            if len(line) > 0:
                first_char = line[0]
            else:
                first_char = ""

            defaults_lines.append(line)

            if (first_char != "#" and first_char != "-" and
                    first_char != " " and
                    first_char != "\r" and
                    first_char != "\n" and
                    first_char != "\t"):
                total_defaults += 1
        file.close()

        self.defaults = "".join(defaults_lines)

        return total_defaults

    def gather_facts(self):
        """
        Return the number of facts.
        """
        facts = []

        for file in self.yaml_files:
            facts += self.gather_facts_list(file)

        unique_facts = list(set(facts))
        self.facts = unique_facts

        return len(unique_facts)

    def gather_facts_list(self, file):
        """
        Return a list of facts.
        """
        facts = []

        contents = utils.file_to_string(os.path.join(self.paths["role"],
                                        file))
        contents = re.sub(r"\s+", "", contents)
        matches = self.regex_facts.findall(contents)

        for match in matches:
            facts.append(match.split(":")[1])

        return facts

    def gather_files(self):
        """
        Return the number of files.
        """
        self.all_files = utils.files_in_path(self.paths["role"])

        return len(self.all_files)

    def gather_lines(self):
        """
        Return the number of lines.
        """
        total_lines = 0
        for file in self.all_files:
            full_path = os.path.join(self.paths["role"], file)

            with open(full_path, "r") as f:
                for line in f:
                    total_lines += 1

            if full_path.endswith(".yml"):
                self.yaml_files.append(full_path)

        return total_lines

    def tally_role_columns(self):
        """
        Sum up all of the stat columns.
        """
        totals = self.report["totals"]
        roles = self.report["roles"]

        totals["dependencies"] = sum(roles[item]
                                     ["total_dependencies"] for item in roles)
        totals["defaults"] = sum(roles[item]
                                 ["total_defaults"] for item in roles)
        totals["facts"] = sum(roles[item]["total_facts"] for item in roles)
        totals["files"] = sum(roles[item]["total_files"] for item in roles)
        totals["lines"] = sum(roles[item]["total_lines"] for item in roles)

    def valid_meta(self, role):
        """
        Return whether or not the meta file being read is valid.
        """
        if os.path.exists(self.paths["meta"]):
            self.meta_dict = utils.yaml_load(self.paths["meta"])

        is_valid = True

        # utils.yaml_load returns False when the file is invalid
        if isinstance(self.meta_dict, bool):
            is_valid = False

            # we need to turn it back into a dict or crazy things happen
            self.meta_dict = {}

        self.meta_dict = self.make_meta_consistent(self.meta_dict)

        if not is_valid:
            self.report["state"]["failed_meta"] += 1
            self.report["state"]["skipped_readme"] += 1
            self.report["roles"][role]["state"] = "failed"

        return is_valid

    def make_meta_consistent(self, meta_dict):
        """
        Return a new meta dict that fixes inconsistencies.
        """
        # if a meta file consists of only --- then this condition fixes
        # a null error were "meta_info" never exists in self.meta_dict
        if meta_dict is None:
            meta_dict = {}

        if "galaxy_info" not in meta_dict:
            meta_dict["galaxy_info"] = {}

        if "platforms" not in meta_dict["galaxy_info"]:
            meta_dict["galaxy_info"]["platforms"] = []

        if "dependencies" not in meta_dict:
            meta_dict["dependencies"] = []

        if "meta_info" not in meta_dict:
            meta_dict["meta_info"] = {}

        return meta_dict

    def write_meta(self, role):
        """
        Potentially write out a new meta file.
        """
        j2_out = self.meta_template.render(self.meta_template_vars)

        # merge the original meta dict's contents into the meta dict
        # this avoids clobbering user edited values in the meta file
        merged_dict = False
        if os.path.exists(self.paths["meta"]):
            merged_dict = True
            original_meta_dict = utils.yaml_load(self.paths["meta"])
            original_meta_dict = self.make_meta_consistent(original_meta_dict)
            self.meta_dict = dict(self.meta_dict.items() +
                                  original_meta_dict.items())

            # make sure certain values are set if they didn't exist previously
            template_meta_dict = utils.yaml_load("", input=j2_out)
            galaxy_author = template_meta_dict["galaxy_info"]["author"]
            galaxy_platforms = template_meta_dict["galaxy_info"]["platforms"]
            meta_github_url = template_meta_dict["meta_info"]["github_url"]

            if len(self.meta_dict["galaxy_info"]["platforms"]) == 0:
                self.meta_dict["galaxy_info"]["platforms"] = galaxy_platforms

            if "author" not in self.meta_dict["galaxy_info"]:
                self.meta_dict["galaxy_info"]["author"] = galaxy_author

            # always renew the github url
            self.meta_dict["meta_info"]["github_url"] = meta_github_url
        else:
            # otherwise just use what we have
            self.meta_dict = utils.yaml_load("", input=j2_out)

        # sort of hacky but works completely fine for now
        self.meta_dict["galaxy_info"]["platforms"] = utils.to_nice_yaml(
            self.meta_dict["galaxy_info"]["platforms"])

        self.meta_dict["dependencies"] = utils.to_nice_yaml(
            self.meta_dict["dependencies"])

        if merged_dict:
            # we need to recompile the template with the merged dict
            j2_out = self.meta_template.render(self.meta_dict)

        self.update_role_report(role, "meta", j2_out)

    def write_readme(self, role):
        """
        Potentially write out a new readme file.
        """
        j2_out = self.readme_template.render(self.readme_template_vars)

        self.update_role_report(role, "readme", j2_out)

    def update_role_report(self, role, report_on, template):
        """
        Update the role state and adjust the state totals.
        """
        state = self.report["state"]
        state_not_skipped = False

        if not os.path.exists(self.paths[report_on]):
            state["ok_" + report_on] += 1
            self.report["roles"][role]["state"] = "ok"
            state_not_skipped = True
        elif self.report["roles"][role][report_on] != template:
            state["changed_" + report_on] += 1
            self.report["roles"][role]["state"] = "changed"
            state_not_skipped = True
        elif self.report["roles"][role][report_on] == template:
            state["skipped_" + report_on] += 1
            self.report["roles"][role]["state"] = "skipped"
            return

        # avoid clobbering non-skipped states in the terminal output
        if state_not_skipped:
            self.report["state"] = state

        utils.string_to_file(self.paths[report_on], template)

    def set_meta_template_vars(self, role):
        """
        Set the meta template variables.
        """
        github_url = "https://github.com/{0}/{1}".format(
            self.options.github_username,
            role.replace("_", "-"))

        self.meta_template_vars = {
            "galaxy_info": {
                "author": utils.capture_shell("git config user.name")[0][:-1],
            },
            "meta_info": {
                "github_url": github_url,
            }
        }

    def set_readme_template_vars(self, role, repo_name):
        """
        Set the readme template variables.
        """
        # at this stage both the platforms and dependencies are strings
        # because they were pretty printed to the meta file

        # this reverts them back to dicts and lists so they can be
        # looped overly properly in the readme file

        # this approach might make you want to vomit glass but it works
        # for the time being until a better approach is found
        platforms = self.meta_dict["galaxy_info"]["platforms"]
        self.meta_dict["galaxy_info"]["platforms"] = utils.yaml_load(
            "", input=platforms)

        self.meta_dict["dependencies"] = utils.yaml_load(
            "", input=self.meta_dict["dependencies"])

        self.readme_template_vars = {
            "github_username": self.options.github_username,
            "repo_name": repo_name,
            "role_name": utils.role_name(role),
            "galaxy_name": role,
            "galaxy_info": self.meta_dict["galaxy_info"],
            "dependencies": self.meta_dict["dependencies"],
            "meta_info": self.meta_dict["meta_info"]
        }

        if ("defaults" not in self.meta_dict["meta_info"] or
                len(self.meta_dict["meta_info"]["defaults"]) == 0):
            # trolled hard by pep8's 79 characters
            r_defaults = self.report["roles"][role]["defaults"]
            self.readme_template_vars["meta_info"]["defaults"] = r_defaults

        if ("facts" not in self.meta_dict["meta_info"] or
                len(self.meta_dict["meta_info"]["facts"]) == 0):
            facts = "\n".join(self.report["roles"][role]["facts"])
            self.readme_template_vars["meta_info"]["facts"] = facts



import errno
import json
import os
import subprocess
import sys
import urllib2
import yaml

from jinja2 import Template

import constants as c
import ui as ui


# ------------------------------------------------------------------------
# file i/o
# ------------------------------------------------------------------------


def mkdir_p(path):
    """
    Emulate the behavior of mkdir -p.
    """
    try:
        os.makedirs(path)
    except OSError as err:
        if err.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            ui.error(c.MESSAGES["path_unmakable"], err)
            sys.exit(1)


def string_to_file(path, input):
    """
    Write a file from a given string.
    """
    mkdir_p(os.path.dirname(path))

    with open(path, "w+") as file:
        file.write(input)


def file_to_string(path):
    """
    Return the contents of a file when given a path.
    """
    if not os.path.exists(path):
        ui.error(c.MESSAGES["path_missing"], path)
        sys.exit(1)

    with open(path, "r") as contents:
        return contents.read()


def dict_to_json(input):
    """
    Return json from a dict.
    """
    return json.dumps(input)


def url_to_string(url):
    """
    Return the contents of a web site url as a string.
    """
    try:
        page = urllib2.urlopen(url)
    except (urllib2.HTTPError, urllib2.URLError) as err:
        ui.error(c.MESSAGES["url_unreachable"], err)
        sys.exit(1)

    return page


def template(source):
    """
    Return a jinja2 template instance from a path or url.
    """
    if "://" in source:
        contents = url_to_string(source)
    else:
        contents = file_to_string(source)

    try:
        return Template(contents)
    except Exception as err:
        file = os.path.basename(source)
        ui.error(c.MESSAGES["template_error"].replace("%file", file), err)
        sys.exit(1)


def files_in_path(path):
    """
    Return a list of all files in a path but exclude git folders.
    """
    aggregated_files = []

    for dir_, _, files in os.walk(path):
        for file in files:
            relative_dir = os.path.relpath(dir_, path)

            if ".git" not in relative_dir:
                relative_file = os.path.join(relative_dir, file)
                aggregated_files.append(relative_file)

    return aggregated_files


def exit_if_path_not_found(path):
    """
    Exit if the path is not found.
    """
    if not os.path.exists(path):
        ui.error(c.MESSAGES["path_missing"], path)
        sys.exit(1)

# ------------------------------------------------------------------------
# yaml
# ------------------------------------------------------------------------


def yaml_load(path, input="", err_quit=False):
    """
    Return a yaml dict from a file or string with error handling.
    """
    try:
        if len(input) > 0:
            return yaml.load(input)
        else:
            return yaml.load(file_to_string(path))
    except Exception as err:
        file = os.path.basename(path)
        ui.error("",
                 c.MESSAGES["yaml_error"].replace("%file", file), err,
                 "")

        if err_quit:
            sys.exit(1)

        return False


def to_nice_yaml(yaml_input, indentation=2):
    """
    Return condensed yaml into human readable yaml.
    """
    return yaml.safe_dump(yaml_input, indent=indentation,
                          allow_unicode=True, default_flow_style=False)


def j2_to_yaml(path):
    """
    Return yaml without the jinja2 template tags.
    """
    yaml_without_j2_tags = []

    file = open(path, "r")
    for line in file:
        if not line.startswith("{"):
            yaml_without_j2_tags.append(line)
    file.close()

    return yaml_load("".join(yaml_without_j2_tags))


# ------------------------------------------------------------------------
# process capturing and detection
# ------------------------------------------------------------------------


def capture_shell(command):
    """
    Return the stdout/stderr from a command.
    """
    proc = subprocess.Popen(command,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE,
                            shell=True)

    return proc.communicate()


# ------------------------------------------------------------------------
# ansigenome specific
# ------------------------------------------------------------------------
def exit_if_no_roles(roles_count, roles_path):
    """
    Exit if there were no roles found.
    """
    if roles_count == 0:
        ui.warn(c.MESSAGES["empty_roles_path"], roles_path)
        sys.exit()


def roles_dict(path, repo_prefix=""):
    """
    Return a dict of role names and repo paths.
    """
    exit_if_path_not_found(path)

    aggregated_roles = {}

    roles = os.walk(path).next()[1]

    for role in roles:
        if is_role(os.path.join(path, role)):
            if isinstance(role, basestring):
                role_url_ready = role_name(role).replace("_", "-")
                role_repo = "{0}{1}".format(repo_prefix, role_url_ready)

                aggregated_roles[role] = role_repo

    return aggregated_roles


def role_name(role):
    """
    Return the role name from a folder name.
    """
    if "." in role:
        return role.split(".")[1]

    return role


def is_role(path):
    """
    Determine if a path is an ansible role.
    """
    seems_legit = False
    for folder in c.ANSIBLE_FOLDERS:
        if os.path.exists(os.path.join(path, folder)):
            seems_legit = True

    return seems_legit


def stripped_args(args):
    """
    Return the stripped version of the arguments.
    """
    stripped_args = []
    for arg in args:
        stripped_args.append(arg.strip())

    return stripped_args



import json
import random
import os
import shutil
import string

import ansigenome.utils as utils


DEFAULTS_TEMPLATE = """---
foo: bar
list:
  one: 1
  two: 2
  three: 3



# comment
 a spaced variable should not be counted
baz: qux
"""

TASKS_TEMPLATE = """---
- set_fact:
  unix_is_cool: "who | grep -i blonde | date; cd ~; unzip; touch"

- set_almost_fact:
  command: "echo 'nope, not quite'"

- set_fact:
  duplicate: "not yet"

- set_fact:
  duplicate: "this should not be counted"
"""

META_TEMPLATE = """---
galaxy_info: {}
dependencies: []
meta_info: {}
"""


def rmrf(path):
    """
    Delete an entire path without warning.
    """
    shutil.rmtree(path)


def random_string(length=8):
    """
    Generate a random string.
    """
    return "".join(random.choice(
        string.ascii_lowercase) for _ in range(length))


def create_roles(path, number=2):
    """
    Create a number of roles.
    """
    role_names = []

    for i in range(number):
        full_path = os.path.join(path, random_string())

        utils.capture_shell("ansigenome init {0}".format(full_path))

        role_names.append(os.path.basename(full_path))

    return role_names


def json_to_dict(input):
    """
    Return a dict from json.
    """
    return json.loads(input)


def print_out(title, out):
    """
    Print the output with borders.
    """
    print
    print "".join("=" * 79)
    print title
    print "".join("=" * 79)
    print out
    print "".join("=" * 79)



import os.path


MODULE_DIR = os.path.join(os.path.dirname(__file__), os.path.pardir)

VALID_ACTIONS = ("scan", "rebuild", "run", "init", "export", "dump")

ANSIBLE_FOLDERS = ("defaults", "handlers", "meta",
                   "tasks", "templates", "tests", "vars")

ROLES_PATH = os.path.join("playbooks", "roles")

README_TEMPLATE_PATH = os.path.join(MODULE_DIR,
                                    "templates", "README.md.j2")
META_TEMPLATE_PATH = os.path.join(MODULE_DIR,
                                  "templates", "meta", "main.yml.j2")
LOG_COLOR = {
    "ok": "green",
    "skipped": "cyan",
    "changed": "yellow",
    "failed": "red"
}

MESSAGES = {
    "default_roles_path_missing": "By default ansigenome will look for " +
    " roles in 'playbooks/roles', that path was not found so you " +
    " must supply a path to your roles\n",
    "empty_roles_path": "No roles were found at this path:",
    "path_missing": "The following path could not be found:",
    "path_exists": "The following path already exists:",
    "path_unmakable": "The following error occurred when trying to " +
    "create this path:",
    "url_unreachable": "The following url was unreachable:",
    "yaml_error": "%file contains 1 or more syntax errors:",
    "template_error": "%file contains 1 or more syntax errors:",
    "github_user_missing": "You must supply --github-user, " +
    "ie. github-user foo",
    "run_success": "%role_count roles were modified with this shell command:",
    "run_error": "There was an error running this shell command:",
    "dump_success": "The roles were dumped as json to:",
    "help_scan": "scan a path containing Ansible roles and report back " +
    "useful stats",
    "help_rebuild": "rebuild a path of roles which auto generates a " +
    "README.md and meta/main.yml from templates",
    "help_run": "run shell commands inside of each role's directory",
    "help_init": "init new roles with a travis-ci test already made for you",
    "help_export": "export a path of roles to a file to be consumed " +
    "by ansible-galaxy install -r",
    "help_dump": "dump a json file containing every stat it gathers from " +
    "a directory"
}

TEST_PATH = os.path.join(os.path.sep, "tmp", "ansigenome")



import os
import utils as utils


default_yml_item = """

- src: 'https://github.com/%github_username/%repo_name'
  version: '%branch'
"""


class Export(object):
    """
    Create a requirements file meant to hold a list of all of your roles.
    """
    def __init__(self, args, options):
        self.roles = utils.roles_dict(args[0])
        self.out_directory = options.out_dir
        self.github_username = options.github_username
        self.branch = options.branch
        self.repo_prefix = options.repo_prefix
        self.out_path = os.path.join(self.out_directory, "requirements")

        if options.yaml:
            self.out_path += ".yml"
            self.export_yml_requirements()
        else:
            self.out_path += ".txt"
            self.export_txt_requirements()

    def export_yml_requirements(self):
        """
        Export the yaml version of the requirements file.

        This only supports github.com for now because the documentation on
        how to use bitbucket seems a bit unstable.
        """
        updated_roles = "---"

        for _, role in self.roles.items():
            if self.repo_prefix:
                role = self.repo_prefix + role

            yml_item = default_yml_item

            yml_item = yml_item.replace("%github_username",
                                        self.github_username)
            yml_item = yml_item.replace("%branch", self.branch)
            yml_item = yml_item.replace("%repo_name", role)

            updated_roles += "{0}".format(yml_item)

        utils.string_to_file(self.out_path, updated_roles)

    def export_txt_requirements(self):
        """
        Export the text version of the requirements file.
        """
        updated_roles = ""

        for role in self.roles.keys():
            updated_roles += "{0},{1}\n".format(role, self.branch)

        utils.string_to_file(self.out_path, updated_roles)



import os
import sys

import constants as c
import ui as ui
import utils as utils


default_mainyml_template = """---
# role: %repo_name

# %values go here
"""

default_travisyml_template = """---
language: "python"
python: "2.7"

env:
  - SITE_AND_INVENTORY="tests/main.yml -i tests/inventory"

install:
  - "pip install ansible"
  - "printf '[defaults]\\nroles_path = ../' > ansible.cfg"

script:
  - "ansible-playbook $SITE_AND_INVENTORY --syntax-check"
  - "ansible-playbook $SITE_AND_INVENTORY --connection=local -vvvv"
  - >
    ansible-playbook $SITE_AND_INVENTORY --connection=local
    | grep -q 'changed=0.*failed=0'
    && (echo 'Idempotence test: PASS' && exit 0)
    || (echo 'Idempotence test: FAIL' && exit 1)
"""

default_travis_main_template = """---
- hosts: localhost
  remote_user: travis
  sudo: true

  roles:
    - %repo_name
"""


class Init(object):
    """
    Create a new role, this is meant to replace ansible-galaxy init.
    """
    def __init__(self, args, options):
        self.output_path = args[0]
        self.role_name = os.path.basename(self.output_path)

        if options.repo_name:
            self.repo_name = options.repo_name
        else:
            self.repo_name = self.role_name

        self.exit_if_path_exists()
        self.create_skeleton()
        self.create_tests()

    def exit_if_path_exists(self):
        """
        Exit early if the path cannot be found.
        """
        if os.path.exists(self.output_path):
            ui.error(c.MESSAGES["path_exists"], self.output_path)
            sys.exit(1)

    def create_skeleton(self):
        """
        Create the role's directory and file structure.
        """
        for folder in c.ANSIBLE_FOLDERS:
            create_folder_path = os.path.join(self.output_path, folder)
            utils.mkdir_p(create_folder_path)

            mainyml_template = default_mainyml_template.replace(
                "%repo_name", self.repo_name)
            mainyml_template = mainyml_template.replace(
                "%values", folder)

            if not folder == "templates" and not folder == "meta":
                utils.string_to_file(os.path.join(create_folder_path,
                                                  "main.yml"),
                                     mainyml_template)

    def create_tests(self):
        """
        Create a travis test case which tests for idempotency.
        """
        utils.string_to_file(os.path.join(self.output_path, ".travis.yml"),
                             default_travisyml_template)
        utils.string_to_file(os.path.join(self.output_path, "tests",
                             "inventory"), "localhost\n")
        utils.string_to_file(os.path.join(self.output_path, "tests",
                             "main.yml"),
                             default_travis_main_template.
                             replace("%repo_name", self.repo_name))
